<?php

if (!defined('ABSPATH'))
    exit;

function Load_BehBank_Gateway()
{

    if (class_exists('WC_Payment_Gateway') && !class_exists('WC_BehBank') && !function_exists('Woocommerce_Add_BehBank_Gateway')) {


        add_filter('woocommerce_payment_gateways', 'Woocommerce_Add_BehBank_Gateway');

        function Woocommerce_Add_BehBank_Gateway($methods)
        {
            $methods[] = 'WC_BehBank';
            return $methods;
        }

        class WC_BehBank extends WC_Payment_Gateway
        {

            public function __construct()
            {

                $this->author = 'behbank.com';
                $this->id = 'WC_BehBank';
                $this->method_title = __('بهبانک', 'woocommerce');
                $this->method_description = __( 'تنظیمات درگاه بهبانک', 'woocommerce');
                $this->icon = apply_filters('WC_GateWay_logo', WP_PLUGIN_URL . "/" . plugin_basename(dirname(__FILE__)) . '/logo.png');
                $this->has_fields = false;

                $this->init_form_fields();
                $this->init_settings();

                $this->title = $this->settings['title'];
                $this->description = $this->settings['description'];
                $this->merchantcode = $this->settings['merchantcode'];

                if ( version_compare( WOOCOMMERCE_VERSION, '2.0.0', '>=' ) ){
                    add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
                }else{
                    add_action( 'woocommerce_update_options_payment_gateways', array( $this, 'process_admin_options' ) );
                }
                add_action('woocommerce_receipt_'.$this->id.'', array($this, 'paymentRequest'));
                add_action('woocommerce_api_'.strtolower(get_class($this)).'', array($this, 'paymentVerify') );


            }


            public function admin_options()
            {
                parent::admin_options();
            }

            public function init_form_fields(){
                $this->form_fields = apply_filters('WC_GateWay_Config',
                    array(
                        'enabled' => array(
                            'title'   => __( 'وضعیت', 'woocommerce' ),
                            'type'    => 'checkbox',
                            'label'   => __( 'درگاه بهبانک فعال باشد', 'woocommerce' ),
                            'description' => __( 'برای فعال سازی درگاه پرداخت بهبانک چک باکس را تیک بزنید', 'woocommerce' ),
                            'default' => 'yes',
                            'desc_tip'    => true,
                        ),
                        'merchantcode' => array(
                            'title'       => __( 'مرچنت کد درگاه', 'woocommerce' ),
                            'type'        => 'text',
                            'description' => __( 'مرچنت کد درگاه بهبانک', 'woocommerce' ),
                            'default'     => '',
                            'desc_tip'    => true
                        ),
                        'title' => array(
                            'title'       => __( 'عنوان درگاه', 'woocommerce' ),
                            'type'        => 'text',
                            'description' => __( 'عنوان درگاه', 'woocommerce' ),
                            'default'     => __( 'بهبانک', 'woocommerce' ),
                            'desc_tip'    => true,
                        ),
                        'description' => array(
                            'title'       => __( 'توضیحات درگاه', 'woocommerce' ),
                            'type'        => 'textarea',
                            'desc_tip'    => true,
                            'description' => __( 'توشیحاتی که در طول انجام عملیات پرداخت نمایش داده میشود', 'woocommerce' ),
                            'default'     => __( 'پرداخت به وسیله کلیه کارتهای عضو شبکه شتاب از طریق سرویس پرداخت بهبانک', 'woocommerce' )
                        ),
                    )
                );
            }

            public function process_payment($order_id)
            {
                $order = new WC_Order($order_id);
                return array(
                    'result' => 'success',
                    'redirect' => $order->get_checkout_payment_url(true)
                );
            }


            public function paymentRequest($order_id){

                if ( ! class_exists( 'GateWay' ) ){
                    require('GateWay.php');
                }


                global $woocommerce;
                $woocommerce->session->order_id_faragate = $order_id;
                $order = new WC_Order( $order_id );
                $currency = $order->get_order_currency();
                $currency = apply_filters( 'WC_GateWay_Currency', $currency, $order_id );
                $form = '<form action="" method="POST" class="gateway-checkout-form" id="gateway-checkout-form">
						<input type="submit" name="gateway_payment" class="button alt" id="gateway-payment-button" value="'.__( 'پرداخت', 'woocommerce' ).'"/>
						<a class="button cancel" href="' . $woocommerce->cart->get_checkout_url() . '">' . __( 'بازگشت', 'woocommerce' ) . '</a>
					 </form><br/>';
                $form = apply_filters( 'WC_GateWay_Form', $form, $order_id, $woocommerce );

                do_action( 'WC_GateWay_Gateway_Before_Form', $order_id, $woocommerce );
                echo $form;
                do_action( 'WC_GateWay_Gateway_After_Form', $order_id, $woocommerce );

                if ( isset($_POST["gateway_payment"]) ) {
                    ob_start();
                    $Amount = intval($order->order_total);
                    $Amount = apply_filters( 'woocommerce_order_amount_total_IRANIAN_gateways_before_check_currency', $Amount, $currency );
                    if ( strtolower($currency) == strtolower('IRT') || strtolower($currency) == strtolower('TOMAN')
                        || strtolower($currency) == strtolower('Iran TOMAN') || strtolower($currency) == strtolower('Iranian TOMAN')
                        || strtolower($currency) == strtolower('Iran-TOMAN') || strtolower($currency) == strtolower('Iranian-TOMAN')
                        || strtolower($currency) == strtolower('Iran_TOMAN') || strtolower($currency) == strtolower('Iranian_TOMAN')
                        || strtolower($currency) == strtolower('تومان') || strtolower($currency) == strtolower('تومان ایران')
                    )
                        $Amount = $Amount*10;
                    else if ( strtolower($currency) == strtolower('IRHT') )
                        $Amount = $Amount*1000*10;
                    else if ( strtolower($currency) == strtolower('IRHR') )
                        $Amount = $Amount*1000;

                    $Amount = apply_filters( 'woocommerce_order_amount_total_IRANIAN_gateways_after_check_currency', $Amount, $currency );
                    $Amount = apply_filters( 'woocommerce_order_amount_total_IRANIAN_gateways_irt', $Amount, $currency );
                    $Amount = apply_filters( 'woocommerce_order_amount_total_FaraGate_gateway', $Amount, $currency );

                    $MerchantCode = $this->merchantcode;
                    $CallbackUrl = add_query_arg( 'wc_order', $order_id , WC()->api_request_url('WC_GateWay') );

                    $products = array();
                    $order_items = $order->get_items();
                    foreach( (array) $order_items as $product ) {
                        $products[] = $product['name'] . ' (' . $product['qty'] . ') ';
                    }
                    $products = implode( ' - ', $products );

                    $Description = 'شماره سفارش : '.$order->get_order_number().' - نام خریدار : '.$order->billing_first_name.' '.$order->billing_last_name . ' - محصولات : ' . $products;
                    $Mobile = get_post_meta( $order_id, '_billing_phone', true ) ? get_post_meta( $order_id, '_billing_phone', true ) : '-';
                    $Email = $order->billing_email;
                    $InvoiceNumber = intval($order->get_order_number());

                    //Hooks for iranian developer
                    $Description = apply_filters( 'WC_GateWay_Description', $Description, $order_id );
                    $Mobile = apply_filters( 'WC_GateWay_Mobile', $Mobile, $order_id );
                    $Email = apply_filters( 'WC_GateWay_Email', $Email, $order_id );
                    $InvoiceNumber = apply_filters( 'WC_GateWay_InvoiceNumber', $InvoiceNumber, $order_id );
                    do_action( 'WC_GateWay_Gateway_Payment', $order_id, $Description, $Mobile );

                    header('Content-Type: text/html; charset=utf-8');
                    header('Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0');
                    header('Pragma: no-cache');

                    $Email = !filter_var($Email, FILTER_VALIDATE_EMAIL) === false ? $Email : '';
                    $Mobile = preg_match('/^09[0-9]{9}/i', $Mobile) ? $Mobile : '';

                    $params = array(
                        'merchantCode'  	  	=> $MerchantCode,
                        'amount'   		  		=> $Amount,
                        'callBackUrl'    		=> $CallbackUrl,
                        'invoiceNumber'		  	=> $InvoiceNumber,
                        'payerEmail' 	  		=> $Email,
                        //'payerMobile' 	  		=> $Mobile,
                        'description'			=> $Description. ' تماس : '.$Mobile
                    );

                    try {
                        $gateWay = new GateWay();
                        $request = $gateWay->paymentRequest($params);
                        if($request['status']){
                            do_action( 'WC_GateWay_Before_Send_to_Gateway', $order_id );
                            ob_end_flush();
                            ob_end_clean();
                            $gateWay->paymentGateway($request['paymentNumber']);
                        }else {
                            $Fault  = 0;
                            $Message = isset($request['message']) ? $request['message'] : '';
                        }
                    }
                    catch(Exception $ex){
                        $Message = $ex->getMessage();
                        $Fault = '';
                    }


                    if ( !empty($Message) && $Message ) {

                        $Note = sprintf( __( 'خطا در هنگام ارسال به بانک : %s', 'woocommerce'), $Message );
                        $Note = apply_filters( 'WC_GateWay_Send_to_Gateway_Failed_Note', $Note, $order_id, $Fault );
                        $order->add_order_note( $Note );


                        $Notice = sprintf( __( 'در هنگام اتصال به بانک خطای زیر رخ داده است : <br/>%s', 'woocommerce'), $Message );
                        $Notice = apply_filters( 'WC_GateWay_Send_to_Gateway_Failed_Notice', $Notice, $order_id, $Fault );
                        if ( $Notice )
                            wc_add_notice( $Notice , 'error' );

                        do_action( 'WC_GateWay_Send_to_Gateway_Failed', $order_id, $Fault );

                    }
                }
            }

            public function paymentVerify(){

                if ( ! class_exists( 'GateWay' ) ){
                    require('GateWay.php');
                }

                $InvoiceNumber = isset($_POST['InvoiceNumber']) ? $_POST['InvoiceNumber'] : '';

                global $woocommerce;
                if ( isset($_GET['wc_order']) )
                    $order_id = $_GET['wc_order'];
                else if ( $InvoiceNumber ) {
                    $order_id = $InvoiceNumber;
                }
                else {
                    $order_id = $woocommerce->session->order_id_faragate;
                    unset($woocommerce->session->order_id_faragate);
                }

                if ( $order_id ) {

                    $order = new WC_Order($order_id);
                    $currency = $order->get_order_currency();
                    $currency = apply_filters( 'WC_GateWay_Currency', $currency, $order_id );

                    if($order->status !='completed'){

                        $MerchantCode = $this->merchantcode;

                        header('Content-Type: text/html; charset=utf-8');
                        header('Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0');
                        header('Pragma: no-cache');

                        $InvoiceNumber = isset($_POST['invoiceNumber']) ? $_POST['invoiceNumber'] : '';
                        $Transaction_ID = $PaymentNumber = isset($_POST['paymentNumber']) ? $_POST['paymentNumber'] : '';

                        if( isset($_POST['status']) && $_POST['status'] == 1 ) {
                            try {
                                $params = array(
                                    'merchantCode' => $MerchantCode,
                                    'paymentNumber' => $PaymentNumber,
                                );
                                $gateWay = new GateWay();;
                                $request = $gateWay->paymentVerify($params);

                                if($request['status']){
                                    $Status = 'completed';
                                    $Fault = '';
                                    $Message = '';
                                }else{
                                    $Status  = 'failed';
                                    $Fault   = 0;
                                    $Message = isset($request['message']) ? $request['message'] : '';
                                }
                            }
                            catch(Exception $ex){
                                $Message = $ex->getMessage();
                            }

                        }
                        else {
                            $Status = 'failed';
                            $Fault = isset($_POST['status']) ? $_POST['status'] : '';
                            $Message = 'تراکنش انجام نشد .';
                        }

                        if ( $Transaction_ID && ( $Transaction_ID !=0 ) )
                            update_post_meta( $order_id, '_transaction_id', $Transaction_ID );

                        if ( $Status == 'completed') {

                            $order->payment_complete($Transaction_ID);
                            $woocommerce->cart->empty_cart();

                            $Note = sprintf( __('پرداخت شما با موفقیت انجام شد .<br/> شماره پرداخت : %s', 'woocommerce' ), $Transaction_ID );
                            $Note = apply_filters( 'WC_GateWay_Return_from_Gateway_Success_Note', $Note, $order_id, $Transaction_ID );
                            if ($Note){
                                $order->add_order_note( $Note , 1 );
                            }

                            $Notice = 'پرداخت شما باموفقیت انجام شد، شماره پرداخت: '.$Transaction_ID;

                            $Notice = apply_filters( 'WC_GateWay_Return_from_Gateway_Success_Notice', $Notice, $order_id, $Transaction_ID );
                            if ($Notice)
                                wc_add_notice( $Notice , 'success' );


                            do_action( 'WC_GateWay_Return_from_Gateway_Success', $order_id, $Transaction_ID );

                            wp_redirect( add_query_arg( 'wc_status', 'success', $this->get_return_url( $order ) ) );
                            exit;
                        }
                        else {

                            $tr_id = ( $Transaction_ID && $Transaction_ID != 0 ) ? ('<br/>شماره پرداخت : '.$Transaction_ID) : '';

                            $Note = sprintf( __( 'خطا در هنگام بازگشت از بانک : %s %s', 'woocommerce'), $Message , $tr_id );

                            $Note = apply_filters( 'WC_GateWay_Return_from_Gateway_Failed_Note', $Note, $order_id, $Transaction_ID, $Fault );
                            if ($Note){
                                $order->add_order_note( $Note , 1 );
                            }

                            $Notice = 'متاسفانه پرداخت انجام نشد، '.$Message;

                            $Notice = apply_filters( 'WC_GateWay_Return_from_Gateway_Failed_Notice', $Notice, $order_id, $Transaction_ID, $Fault );
                            if ($Notice)
                                wc_add_notice( $Notice , 'error' );

                            do_action( 'WC_GateWay_Return_from_Gateway_Failed', $order_id, $Transaction_ID, $Fault );

                            wp_redirect(  $woocommerce->cart->get_checkout_url()  );
                            exit;
                        }


                    }
                    else {
                        $Transaction_ID = get_post_meta( $order_id, '_transaction_id', true );

                        $Notice = 'پرداخت شما با موفقیت انجام شد، شماره پرداخت : '.$Transaction_ID;

                        $Notice = apply_filters( 'WC_GateWay_Return_from_Gateway_ReSuccess_Notice', $Notice, $order_id, $Transaction_ID );
                        if ($Notice)
                            wc_add_notice( $Notice , 'success' );


                        do_action( 'WC_GateWay_Return_from_Gateway_ReSuccess', $order_id, $Transaction_ID );

                        wp_redirect( add_query_arg( 'wc_status', 'success', $this->get_return_url( $order ) ) );
                        exit;
                    }
                }
                else {

                    $Fault = __('شماره سفارش وجود ندارد .', 'woocommerce' );
                    $Notice = 'متاسفانه پرداخت انجام نشد ، '.$Fault;
                    $Notice = apply_filters( 'WC_GateWay_Return_from_Gateway_No_Order_ID_Notice', $Notice, $order_id, $Fault );
                    if ($Notice)
                        wc_add_notice( $Notice , 'error' );

                    do_action( 'WC_GateWay_Return_from_Gateway_No_Order_ID', $order_id, '', $Fault );

                    wp_redirect(  $woocommerce->cart->get_checkout_url()  );
                    exit;
                }
            }

        }

    }
}

add_action('plugins_loaded', 'Load_BehBank_Gateway', 0);
